# README
<!-- TOC -->

- [README](#readme)
  - [Overview](#overview)
  - [Start development](#start-development)
    - [Download dependency](#download-dependency)
    - [Compile the war file](#compile-the-war-file)
    - [Deploy war](#deploy-war)
  - [Stop development](#stop-development)
  - [General Docker commands](#general-docker-commands)
  - [Advanced Docker commands](#advanced-docker-commands)
    - [Clean commands](#clean-commands)
    - [Monitoring commands](#monitoring-commands)

<!-- /TOC -->

## Overview

This file provides commands to start development with Docker.

Requires:
 - Docker installed.
 - Docker well configured, cf. **/etc/docker/deamon.json**
 - User in `docker` group to avoid sudo/su right

## Start development

### Download dependency

Go to the **root directory** of this project and download geonetwork.war

```bash
mkdir -p source
cd source
wget --no-check-certificate -O geonetwork-4.4.1-0.war https://sourceforge.net/projects/geonetwork/files/GeoNetwork_opensource/v4.4.1/geonetwork.war/download
chmod o+r *.war
cd ..
```

### Compile the war file

Issue these commands in the root directory of this project:

```bash
docker run -u root -w "/build" --volume "${PWD}:/build" docker.alkante.com/alk-tomcat:9java8 bash -c "umask 000 && ./mvnw install:install-file -DgroupId=org.geonetwork-opensource -DartifactId=geonetwork -Dversion=4.4.1-0 -Dpackaging=war -Dfile=./source/geonetwork-4.4.1-0.war && ./mvnw clean package"
```

### Requirements

 - Start Elasticsearch according to [Alkante](https://gitlab.alkante.com/alkante/elasticsearch) or local documentation.
 - Start `pr_prodige` according to [its documentation](https://gitlab.adullact.net/prodige/pr_prodige/-/blob/master/documentation/installation.md)

### Adapt parameters to your local config

Dans ./jpr_prodige_geonetwork/cicd/dev :


```bash
sudo su

cat > ../../../pr_prodige/data/geonetwork/WEB-INF/config.properties << CONFIG
language.default=eng
language.forceDefault=false


# Define using CRON expression when watchlist notifier is triggered (Default 4AM)
usersavedselection.watchlist.frequency=0 0 4 * * ?

# Define the link to the list of updated records sent by email by the watchlist notifier
usersavedselection.watchlist.searchurl=catalog.search#/search?_uuid={{filter}}

# Define the link to each record sent by email by the watchlist notifier
usersavedselection.watchlist.recordurl=api/records/{{index:uuid}}

es.protocol=http
es.port=9200
es.host=$(echo $(host $(hostname -f)) | cut -d ' ' -f 4)
CONFIG

cat >> ../../../pr_prodige/data/geonetwork/WEB-INF/config.properties << 'CONFIG'
es.url=${es.protocol}://${es.host}:${es.port}
es.username=
es.password=
es.index.features=gn-features
es.index.features.type=features
# Define the number of decimals to apply when converting geometries to GeoJSON
# Too high value means more volume. Adapt the value depending on the precision
# of the WFS harvested which usually allows to define a number of decimals
# in GetFeature responses.
es.index.features.numberOfDecimals=8
# Force reduction of geometry precision based on the number of decimals
es.index.features.applyPrecisionModel=true
es.index.features.featureCommitInterval=250
es.index.records=gn-records
es.index.records.type=records
es.index.records_public=${es.index.records_public}
es.index.searchlogs=gn-searchlogs
es.index.searchlogs.type=searchlogs

kb.url=http://localhost:5601

es.index.checker.interval=0/5 * * * * ?

jms.url=tcp://localhost:61616

bot.regexpFilter=@bot.regexpFilter@

api.params.maxPageSize=20000
api.params.maxUploadSize=100000000
urlChecker.UserAgent=GeoNetwork Link Checker

map.bbox.background.service=https://ows.terrestris.de/osm/service?SERVICE=WMS&amp;REQUEST=GetMap&amp;VERSION=1.1.0&amp;LAYERS=OSM-WMS&amp;STYLES=default&amp;SRS={srs}&amp;BBOX={minx},{miny},{maxx},{maxy}&amp;WIDTH={width}&amp;HEIGHT={height}&amp;FORMAT=image/png

# Set to false to enable the services to draw map extents (region.getmap and {metadatauuid}/extents.png) accepting
# urls for map services to provide the background layers. Otherwise the only allowed options are from the settings
# configuration or a named bg layer from regionGetMapBackgroundLayers.
metadata.extentApi.disableFullUrlBackgroundMapServices=true
CONFIG

chown www-data:www-data ../../../pr_prodige/data/geonetwork/WEB-INF/config.properties
```

### Deploy war

#### In `pr_prodige` directory

 - Stop `jpr_prodige_geonetwork_web`

#### In this `./cicd/dev` directory: run container

Issue these commands in this directory: `cd ./cicd/dev`

```bash
# Build dev image
docker-compose build

# Run containers
docker-compose up
```

[URL](https://catalogue.prodige.internal/geonetwork/)

## simplified procedure to rebuild

```bash
bash reload.sh
```

## Stop development

Issue these commands in this directory: `cd ./cicd/dev`

```bash
# Shutdown containers
docker-compose down

# Remove dev image
docker rmi jpr_prodige_geonetwork_dev_web
```

## General Docker commands

| Description                    | Commandes |
| ------------------------------ | ------------------------- |
| Display images            | `docker images` |
| Remove image              | `docker rmi myimage` |
| ------------------------------ | ------------------------- |
| Display all running containers | ```docker ps``` |
| Display all containers    | ```docker ps -a``` |
| Remove container          | ```docker rm mycontainer``` |
| Display info container    | ```docker inpect mycontainer``` |
| Display container output       | ```docker logs mycontainer``` |
| Stop container            | ```docker stop mycontainer``` |
| Kill container            | ```docker kill mycontainer``` |
| ------------------------------ | ------------------------- |
| Connection as root | ```docker exec --user 0 -w / -it jpr_prodige_geonetwork_dev_web bash``` |
| Connection as www-data | ```docker exec --user www-data -w /var/www/html -it jpr_prodige_geonetwork_dev_web bash``` |
| Connection as postgres | ```docker exec --user postgres -it tpl_alkante_symfony_3_dev_db bash``` |
| Run composer install | ```docker exec --user www-data -w /var/www/html -it jpr_prodige_geonetwork_dev_web composer install ``` |

## Advanced Docker commands


### Clean commands
```bash
# Stop all container
docker stop $(docker ps -a -q)

# Remove all container
docker rm $(docker ps -a -q)

# Delete all image with name or tag aas "<none>"
docker rmi `docker images| egrep "<none>" |awk '{print $3}'`
```

### Monitoring commands

```bash
# Watch all container
watch -n 1 "docker ps -a"

# Watch all images
watch -n 1 "docker images"

# Watch all resources container
docker stats
```
